@@ -22,8 +22,6 @@ from astropy import coordinates
 from astropy import table
 from astropy.io import ascii
 from astropy import stats
-from astropy import units as u
-from astropy import constants as const
 
 from linetools.spectra.xspectrum1d import XSpectrum1D
 
@@ -34,7 +32,7 @@ from pypeit import io
 from pypeit.wavemodel import conv2res
 from pypeit.core.wavecal import wvutils
 from pypeit.core import fitting
-from pypeit.core import telluric
+#from pypeit.core import telluric
 
 
 # TODO: Put these in the relevant functions
@@ -42,23 +40,56 @@ TINY = 1e-15
 SN2_MAX = (20.0) ** 2
 PYPEIT_FLUX_SCALE = 1e-17
 
+
 def zp_unit_const():
     """
-    This constant defines the units for the spectroscopic zeropoint. See the doc/dev/fluxing.rst doc for more information.
-    Returns
-    -------
-
+    This constant defines the units for the spectroscopic zeropoint. See
+    :ref:`fluxcalib`.
     """
-    return -2.5*np.log10(((u.angstrom**2/const.c)*(PYPEIT_FLUX_SCALE*u.erg/u.s/u.cm**2/u.angstrom)).to('Jy')/(3631 * u.Jy)).value
+    return -2.5*np.log10(((units.angstrom**2/constants.c) * 
+                          (PYPEIT_FLUX_SCALE*units.erg/units.s/units.cm**2/units.angstrom)
+                         ).to('Jy')/(3631 * units.Jy)).value
+
 
 # This function is defined to convert AB magnitudes to cgs unit erg cm^-2 s^-1 A^-1
 def mAB_to_cgs(mAB,wvl):
     return 10**((-48.6-mAB)/2.5)*3*10**18/wvl**2
 
-# Define this global variable to avoid constantly recomputing, which could be costly in the telluric optimization routines.
-# It has a value of ZP_UNIT_CONST = 40.092117379602044
+
+def blackbody_func(a, teff):
+    """
+    Generate a blackbody spectrum based on the normalisation and effective temperature.
+    See Suzuki & Fukugita, 2018, AJ, 156, 219:
+    https://ui.adsabs.harvard.edu/abs/2018AJ....156..219S/abstract
+
+    Args:
+        a (float):
+            flux normalisation factor
+        teff (float):
+            Effective temperature of the blackbody
+
+    Returns:
+        waves : `numpy.ndarray`_ of the wavelengths
+        flam : `numpy.ndarray`_ flux in units of erg/s/cm^2/A
+    """
+    waves = np.arange(3000.0, 25000.0, 0.1) * units.AA
+    # Setup the units
+    # TODO: This alters the input!!
+    teff *= units.K
+    a *= 1.0E-23
+    # Calculate the function
+    flam = ((a*2*constants.h*constants.c**2)/waves**5) / (np.exp((constants.h*constants.c / 
+                (waves*constants.k_B*teff)).to(units.m/units.m).value)-1.0)
+    flam = flam.to(units.erg / units.s / units.cm ** 2 / units.AA).value / PYPEIT_FLUX_SCALE
+    return waves.value, flam
+
+
+# Define this global variable to avoid constantly recomputing, which could be
+# costly in the telluric optimization routines.  It has a value of ZP_UNIT_CONST
+# = 40.092117379602044
 ZP_UNIT_CONST = zp_unit_const()
 
+
 def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
     """
     Find a match for the input file to one of the archived
@@ -85,12 +116,12 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
 
             - 'cal_file': str -- Filename table
             - 'name': str -- Star name
-            - 'std_ra': str -- RA(J2000)
-            - 'std_dec': str -- DEC(J2000)
+            - 'std_ra': float -- RA(J2000)
+            - 'std_dec': float -- DEC(J2000)
 
     """
     # Priority
-    std_sets = ['xshooter', 'calspec', 'esofil', 'noao']
+    std_sets = ['blackbody', 'xshooter', 'calspec', 'esofil', 'noao']
 
     # SkyCoord
     obj_coord = coordinates.SkyCoord(ra, dec, unit='deg')
@@ -119,8 +150,11 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
             _idx = int(idx)
             std_dict = dict(cal_file=os.path.join(path, star_tbl[_idx]['File']),
                             name=star_tbl[_idx]['Name'],
-                            std_ra=star_tbl[_idx]['RA_2000'],
-                            std_dec=star_tbl[_idx]['DEC_2000'])
+#                            std_ra=star_tbl[_idx]['RA_2000'],
+#                            std_dec=star_tbl[_idx]['DEC_2000'])
+                            # Force the coordinates to be decimal degrees
+                            std_ra=star_coords.ra[_idx].value,
+                            std_dec=star_coords.dec[_idx].value)
 
             if not os.path.isfile(star_file):
                 # TODO: Error or warn?
@@ -150,6 +184,9 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
                 std_dict['flux'] = std_spec['FLUX'] / PYPEIT_FLUX_SCALE \
                                    * units.erg / units.s / units.cm ** 2 / units.AA
             elif sset == 'esofil':
+                fil_basename = os.path.basename(fil)
+                if not fil_basename.startswith('f'):
+                    msgs.error("The ESO reference standard filename must start with the string `f`;  make sure it is the case. Also make sure that the flux units in the file are in 10**(-16) erg/s/cm2/AA.")
                 # TODO let's add the star_mag here and get a uniform set of tags in the std_dict
                 std_spec = table.Table.read(fil, format='ascii')
                 std_dict['std_source'] = sset
@@ -158,9 +195,8 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
                                    units.erg / units.s / units.cm ** 2 / units.AA
                 # At this low resolution, best to throw out entries affected by A and B-band absorption
                 mask = (std_dict['wave'].value > 7551.) & (std_dict['wave'].value < 7749.)
-                std_dict['wave'] = std_dict['wave'][np.invert(mask)]
-                std_dict['flux'] = std_dict['flux'][np.invert(mask)]
-            
+                std_dict['wave'] = std_dict['wave'][np.logical_not(mask)]
+                std_dict['flux'] = std_dict['flux'][np.logical_not(mask)]
             elif sset == 'noao': #mostly copied from 'esofil', need to convert the flux units
                 # TODO let's add the star_mag here and get a uniform set of tags in the std_dict
                 std_spec = table.Table.read(fil, format='ascii')
@@ -170,9 +206,14 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
                                    units.erg / units.s / units.cm ** 2 / units.AA
                 # At this low resolution, best to throw out entries affected by A and B-band absorption
                 mask = (std_dict['wave'].value > 7551.) & (std_dict['wave'].value < 7749.)
-                std_dict['wave'] = std_dict['wave'][np.invert(mask)]
-                std_dict['flux'] = std_dict['flux'][np.invert(mask)]
-            
+                std_dict['wave'] = std_dict['wave'][np.logical_not(mask)]
+                std_dict['flux'] = std_dict['flux'][np.logical_not(mask)]
+            elif sset == 'blackbody':
+                # TODO let's add the star_mag here and get a uniform set of tags in the std_dict
+                waves, flam = blackbody_func(star_tbl[_idx]['a_x10m23'], star_tbl[_idx]['T_K'])
+                std_dict['std_source'] = sset
+                std_dict['wave'] = waves * units.AA
+                std_dict['flux'] = flam * units.erg / units.s / units.cm ** 2 / units.AA
             else:
                 msgs.error('Do not know how to parse {0} file.'.format(sset))
             msgs.info("Fluxes are flambda, normalized to 1e-17")
@@ -185,9 +226,13 @@ def find_standard_file(ra, dec, toler=20.*units.arcmin, check=False):
             closest['sep'] = mind2d
             # TODO: Is this right? Do we need to use the imind2d from
             # above?
-            closest.update(dict(name=star_tbl[int(idx)]['Name'],
-                                ra=star_tbl[int(idx)]['RA_2000'],
-                                dec=star_tbl[int(idx)]['DEC_2000']))
+            _idx = int(idx)
+            closest.update(dict(name=star_tbl[_idx]['Name'],
+#                                ra=star_tbl[int(idx)]['RA_2000'],
+#                                dec=star_tbl[int(idx)]['DEC_2000']))
+                                # Force the coordinates to be decimal degrees
+                                std_ra=star_coords.ra[_idx].value,
+                                std_dec=star_coords.dec[_idx].value))
 
     # Standard star not found
     if check:
@@ -322,7 +367,8 @@ def get_standard_spectrum(star_type=None, star_mag=None, ra=None, dec=None):
             ## Vega model from TSPECTOOL
             vega_file = resource_filename('pypeit', '/data/standards/vega_tspectool_vacuum.dat')
             vega_data = table.Table.read(vega_file, comment='#', format='ascii')
-            std_dict = dict(cal_file='vega_tspectool_vacuum', name=star_type, Vmag=star_mag, std_ra=ra, std_dec=dec)
+            std_dict = dict(cal_file='vega_tspectool_vacuum', name=star_type, Vmag=star_mag,
+                            std_ra=ra, std_dec=dec)
             std_dict['std_source'] = 'VEGA'
             std_dict['wave'] = vega_data['col1'] * units.AA
 
@@ -350,6 +396,7 @@ def load_extinction_data(longitude, latitude, toler=5. * units.deg):
 
     Parameters
     ----------
+    longitude, latitude: Geocentric coordinates in degrees (floats).
     toler : Angle, optional
         Tolerance for matching detector to site (5 deg)
 
@@ -392,8 +439,8 @@ def extinction_correction(wave, airmass, extinct):
     Parameters
     ----------
     wave (`numpy.ndarray`_):
-        Wavelengths for interpolation. Should be sorted Assumes
-        Angstroms
+        Wavelengths for interpolation. Should be sorted.
+        Assumes angstroms.
     airmass : float
         Airmass
     extinct : Table
@@ -402,7 +449,10 @@ def extinction_correction(wave, airmass, extinct):
     Returns:
     -------
     `numpy.ndarray`_:
-        Flux corrections at the input wavelengths
+        Multiplucative flux correction factors
+        at the input wavelengths.
+        i.e. true_flux = correction_factor*observed_flux
+
     """
     # Checks
     if airmass < 1.:
@@ -578,6 +628,7 @@ def sensfunc(wave, counts, counts_ivar, counts_mask, exptime, airmass, std_dict,
     out_table = table.Table(meta={'name': 'Sensitivity Function'})
     # These are transposed because we need to store them in an astropy table, with number of rows = norders
     out_table['SENS_WAVE'] = wave_arr.T
+    out_table['SENS_COUNTS_PER_ANG'] = counts_arr.T
     out_table['SENS_ZEROPOINT'] = zeropoint_data.T
     out_table['SENS_ZEROPOINT_GPM'] = zeropoint_data_gpm.T
     out_table['SENS_ZEROPOINT_FIT'] = zeropoint_fit.T
@@ -627,22 +678,29 @@ def get_sensfunc_factor(wave, wave_zp, zeropoint, exptime, tellmodel=None, extin
     zeropoint_obs = np.zeros_like(wave)
     wave_mask = wave > 1.0  # filter out masked regions or bad wavelengths
     delta_wave = wvutils.get_delta_wave(wave, wave_mask)
+
+#    print(f'get_sensfunc_factor: {np.amin(wave_zp):.1f}, {np.amax(wave_zp):.1f}, '
+#          f'{np.amin(wave[wave_mask]):.1f}, {np.amax(wave[wave_mask]):.1f}')
+
     try:
-        zeropoint_obs[wave_mask] = interpolate.interp1d(wave_zp, zeropoint, bounds_error=True)(wave[wave_mask])
+        zeropoint_obs[wave_mask] \
+                = interpolate.interp1d(wave_zp, zeropoint, bounds_error=True)(wave[wave_mask])
     except ValueError:
         if extrap_sens:
-            zeropoint_obs[wave_mask] = interpolate.interp1d(wave_zp, zeropoint, bounds_error=False)(wave[wave_mask])
-            msgs.warn(
-                "Your data extends beyond the bounds of your sensfunc. You should be adjusting "
-                "the par['sensfunc']['extrap_blu'] and/or par['sensfunc']['extrap_red'] to extrapolate further "
-                "and recreate your sensfunc. But we are extrapolating per your direction. Good luck!")
+            zeropoint_obs[wave_mask] \
+                    = interpolate.interp1d(wave_zp, zeropoint, bounds_error=False)(wave[wave_mask])
+            msgs.warn("Your data extends beyond the bounds of your sensfunc. You should be "
+                      "adjusting the par['sensfunc']['extrap_blu'] and/or "
+                      "par['sensfunc']['extrap_red'] to extrapolate further and recreate your "
+                      "sensfunc. But we are extrapolating per your direction. Good luck!")
         else:
             msgs.error("Your data extends beyond the bounds of your sensfunc. " + msgs.newline() +
-                       "Adjust the par['sensfunc']['extrap_blu'] and/or par['sensfunc']['extrap_red'] to extrapolate "
-                       "further and recreate your sensfunc.")
+                       "Adjust the par['sensfunc']['extrap_blu'] and/or "
+                       "par['sensfunc']['extrap_red'] to extrapolate further and recreate "
+                       "your sensfunc.")
 
-    # This is the S_lam factor required to convert N_lam = counts/sec/Ang to F_lam = 1e-17 erg/s/cm^2/Ang, i.e.
-    # F_lam = S_lam*N_lam
+    # This is the S_lam factor required to convert N_lam = counts/sec/Ang to
+    # F_lam = 1e-17 erg/s/cm^2/Ang, i.e.  F_lam = S_lam*N_lam
     sensfunc_obs = Nlam_to_Flam(wave, zeropoint_obs)
 
     # TODO Telluric corrections via this method are deprecated
@@ -1035,14 +1093,14 @@ def zeropoint_to_throughput(wave, zeropoint, eff_aperture):
 
     """
 
-    eff_aperture_m2 = eff_aperture*u.m**2
-    S_lam_units = 1e-17*u.erg/u.cm**2
+    eff_aperture_m2 = eff_aperture*units.m**2
+    S_lam_units = 1e-17*units.erg/units.cm**2
     # Set the throughput to be -1 in places where it is not defined.
     throughput = np.full_like(zeropoint, -1.0)
     zeropoint_gpm = (zeropoint > 5.0) & (zeropoint < 30.0) & (wave > 1.0)
     inv_S_lam = Flam_to_Nlam(wave[zeropoint_gpm], zeropoint[zeropoint_gpm])/S_lam_units
-    inv_wave = utils.inverse(wave[zeropoint_gpm])/u.angstrom
-    thru = ((const.h*const.c)*inv_wave/eff_aperture_m2*inv_S_lam).decompose()
+    inv_wave = utils.inverse(wave[zeropoint_gpm])/units.angstrom
+    thru = ((constants.h*constants.c)*inv_wave/eff_aperture_m2*inv_S_lam).decompose()
     throughput[zeropoint_gpm] = thru
     return throughput
 
