@@ -2,6 +2,10 @@ import os
 import copy
 import numpy as np
 import matplotlib
+try:
+    matplotlib.use('Qt5Agg')
+except:
+    pass
 import matplotlib.pyplot as plt
 from matplotlib.lines import Line2D
 from matplotlib.colors import LinearSegmentedColormap, Normalize
@@ -30,6 +34,10 @@ operations = dict({'cursor': "Select lines (LMB click)\n" +
                    'f' : "Fit the wavelength solution",
                    'g' : "Toggle ghost solution (show predicted line positions when wavelength is on the x-axis)",
                    'h' : "Reset ghost parameters",
+                   'i' : "Include an undetected line to the detected line list\n" +
+                         "         First select fitting pixels (LMB drag = add, RMB drag = remove)\n" +
+                         "         Then press 'i' to perform a fit." +
+                         "         NOTE: ghost solution must be turned off to select fit regions.",
                    'l' : "Load saved line IDs from file (waveids.ascii in local directory)",
                    'm' : "Select a line",
                    'r' : "Refit a line",
@@ -49,7 +57,7 @@ class Identify(object):
     """
 
     def __init__(self, canvas, axes, spec, specres, detns, line_lists, par, lflag_color,
-                 slit=0, spatid='0', wv_calib=None, pxtoler=None):
+                 slit=0, spatid='0', wv_calib=None, pxtoler=None, specname=""):
         """Controls for the Identify task in PypeIt.
 
         The main goal of this routine is to interactively identify arc lines
@@ -174,7 +182,7 @@ class Identify(object):
         self.replot()
 
     @classmethod
-    def initialise(cls, arccen, slits, slit=0, par=None, wv_calib_all=None,
+    def initialise(cls, arccen, lamps, slits, slit=0, par=None, wv_calib_all=None,
                    wavelim=None, nonlinear_counts=None, test=False,
                    pxtoler=0.1, fwhm=4.):
         """Initialise the 'Identify' window for real-time wavelength calibration
@@ -187,6 +195,9 @@ class Identify(object):
         ----------
         arccen : ndarray
             Arc spectrum
+        lamps : :obj:`list`
+            List of arc lamps to be used for wavelength calibration.
+            E.g., ['ArI','NeI','KrI','XeI']
         slits : :class:`SlitTraceSet`
             Data container with slit trace information
         slit : int, optional
@@ -202,6 +213,7 @@ class Identify(object):
         nonlinear_counts : float, optional
             Counts where the arc is presumed to go non-linear
             Passed to arc_lines_from_spec()
+            Defaults to 1e10 if None is input
         fwhm : float, optional
             FWHM of arc lines in pixels
         pxtoler : float, optional
@@ -223,6 +235,8 @@ class Identify(object):
 
         # Extract the lines that are detected in arccen
         thisarc = arccen[:, slit]
+        if nonlinear_counts is None:
+            nonlinear_counts = 1e10
         tdetns, _, _, icut, _ = wvutils.arc_lines_from_spec(thisarc,
                                                             fwhm=fwhm,
                                                             sigdetect=par['sigdetect'],
@@ -230,11 +244,11 @@ class Identify(object):
         detns = tdetns[icut]
 
         # Load line lists
-        if 'ThAr' in par['lamps']:
-            line_lists_all = waveio.load_line_lists(par['lamps'])
+        if 'ThAr' in lamps:
+            line_lists_all = waveio.load_line_lists(lamps)
             line_lists = line_lists_all[np.where(line_lists_all['ion'] != 'UNKNWN')]
         else:
-            line_lists = waveio.load_line_lists(par['lamps'])
+            line_lists = waveio.load_line_lists(lamps)
 
         # Trim the wavelength scale if requested
         if wavelim is not None:
@@ -640,7 +654,7 @@ class Identify(object):
                 wvcalib = copy.deepcopy(final_fit)
         return wvcalib
 
-    def store_solution(self, final_fit, master_dir, binspec, rmstol=0.15,
+    def store_solution(self, final_fit, binspec, rmstol=0.15,
                        force_save=False, wvcalib=None):
         """Check if the user wants to store this solution in the reid arxiv
 
@@ -649,8 +663,6 @@ class Identify(object):
 
         final_fit : dict
             Dict of wavelength calibration solutions (see self.get_results())
-        master_dir : str
-            Master directory -- NOT USED
         binspec : int
             Spectral binning
         rmstol : float
@@ -695,14 +707,30 @@ class Identify(object):
                 #          msgs.newline() + templates.outpath + msgs.newline() + msgs.newline() +
                 #          "Please consider sending your solution to the PypeIt team!" + msgs.newline())
                 #
+                outfname = "wvcalib.fits"
                 if wvcalib is not None:
-                    wvcalib.to_file('wvcalib.fits')
+                    wvcalib.to_file(outfname, overwrite=True)
                     msgs.info("\nA WaveCalib container was written to wvcalib.fits")
-                msgs.info("\nPlease consider sending your solution to the PypeIt team!" + msgs.newline())
+                print("\n\nPlease visit the following site if you want to include your solution in PypeIt:")
+                print("https://pypeit.readthedocs.io/en/latest/construct_template.html#creating-the-template\n")
+                print("You will need the following information:")
+                print("  (1) spectral binning = {0:d}".format(binspec))
+                print("  (2) slit spat_id = {0:s}".format(self._spatid))
+                print("  (3) the {0:s} file".format(outfname))
+                print("\nPlease consider sending your solution to the PypeIt team!\n")
         else:
-            print("Final fit RMS: {0:0.3f} is larger than the allowed tolerance: {1:0.3f}".format(final_fit['rms'], rmstol))
-            print("Set the variable --rmstol on the command line to allow a more flexible RMS tolerance")
-            ans = ''
+            print("\nFinal fit RMS: {0:0.3f} is larger than the allowed tolerance: {1:0.3f}".format(final_fit['rms'], rmstol))
+            print("Set the variable --rmstol on the command line to allow a more flexible RMS tolerance\n")
+            if ans != 'y':
+                # If we make it here, the user has not chosen to save the IDs, and the rms tol was bad
+                ans = ''
+                if not force_save:
+                    while ans != 'y' and ans != 'n':
+                        ans = input("A solution has not been saved - would you like to write the IDs to disk? (y/n): ")
+                else:
+                    ans = 'y'
+                if ans == 'y':
+                    self.save_IDs()
 
     def button_press_callback(self, event):
         """What to do when the mouse button is pressed
@@ -873,6 +901,9 @@ class Identify(object):
         elif key == 'f':
             self.fitsol_fit()
             self.replot()
+        elif key == 'i':
+            self.add_new_detection()
+            self.replot()
         elif key == 'l':
             self.load_IDs()
         elif key == 'm':
@@ -1019,6 +1050,49 @@ class Identify(object):
             msgs.bug("Cannot predict wavelength value - no fit has been performed")
             return None
 
+    def add_new_detection(self):
+        """
+        Perform a local Gaussian fit to the pixels near the mouse cursor.
+        If the fit is accetable, a new line will be added to the detections
+        provided that the line is not too close to another line already in
+        the detections list.
+        """
+        # Define the minimum distance between lines (in pixels)
+        mindist = 4
+        # Get the selected regions
+        ww = np.where(self._fitregions == 1)
+        xfit = self.specx.copy()[ww]
+        yfit = self.specdata.copy()[ww]
+        from scipy.optimize import curve_fit
+        # Make sure there are enough pixels for the fit
+        npix = len(xfit)
+        if npix <= 3:
+            return
+        # Some starting parameters
+        ampl = np.max(yfit)
+        mean = sum(xfit * yfit) / sum(yfit)
+        sigma = sum(yfit * (xfit - mean) ** 2) / sum(yfit)
+        # Perform a gaussian fit
+        gaus = lambda x, a, x0, sigma: a * np.exp(-(x - x0) ** 2 / (2 * sigma ** 2))
+        popt, pcov = curve_fit(gaus, xfit, yfit, p0=[ampl, mean, sigma])
+        print(ampl, mean, sigma, popt[1])
+        # Get the new detection
+        new_detn = popt[1]
+        # Check that the detection doesn't already exist
+        cls_line = np.min(np.abs(self._detns - new_detn))
+        if cls_line > mindist:
+            detns = np.append(self._detns, new_detn)
+            arsrt = np.argsort(detns)
+            self._detns = detns[arsrt]
+            self._detnsy = self.get_ann_ypos()  # Get the y locations of the annotations
+            self._lineids = np.append(self._lineids, 0)[arsrt]
+            self._lineflg = np.append(self._lineflg, 0)[arsrt]  # Flags: 0=no ID, 1=user ID, 2=auto ID, 3=flag reject
+        else:
+            self.update_infobox("New detection is <{0:d} pixels of a detection - ignoring".format(mindist))
+        # Reset the fit regions
+        self._fitregions = np.zeros_like(self._fitregions)
+        return
+
     def fitsol_fit(self):
         """Perform a fit to the line identifications
         """
@@ -1051,7 +1125,7 @@ class Identify(object):
                     n_final=self._fitdict["polyorder"],
                     sigrej_first=self.par['sigrej_first'],
                     sigrej_final=self.par['sigrej_final'])
-                final_fit.spat_id = self._slit
+                final_fit.spat_id = int(self._spatid)
 
                 # Update the fitdict
                 #for key in final_fit:
@@ -1080,10 +1154,10 @@ class Identify(object):
         Args:
             message (str): Message to be displayed
         """
+
         self.axes['info'].clear()
         if default:
-            self.axes['info'].text(0.5, 0.5, "Press '?' to list the available options", transform=self.axes['info'].transAxes,
-                          horizontalalignment='center', verticalalignment='center')
+            self.axes['info'].text(0.5, 0.5, "Press '?' to list the available options", transform=self.axes['info'].transAxes, horizontalalignment='center', verticalalignment='center')
             self.canvas.draw()
             return
         # Display the message
@@ -1153,9 +1227,10 @@ class Identify(object):
             self._lineids = data['wavelength'].data
             self._lineflg = data['flag'].data
             msgs.info("Loaded line IDs:" + msgs.newline() + fname)
-            self.update_infobox(message="Loaded line IDs: {0:s}".format(fname), yesno=False)
         else:
-            self.update_infobox(message="Could not find line IDs: {0:s}".format(fname), yesno=False)
+            msgs.info("Could not find line IDs:" + msgs.newline()+fname)
+        self._detnsy = self.get_ann_ypos()  # Get the y locations of the annotations
+        self.replot()
 
     def save_IDs(self, fname='waveid.ascii'):
         """Save the current IDs
