@@ -1,4 +1,4 @@
-
+from pypeit.utils import all_subclasses
 from pypeit.spectrographs import spectrograph
 
 # The import of all the spectrograph modules here is what enables the dynamic
@@ -17,6 +17,7 @@ from pypeit.spectrographs import keck_nires
 from pypeit.spectrographs import keck_nirspec
 from pypeit.spectrographs import lbt_luci
 from pypeit.spectrographs import lbt_mods
+from pypeit.spectrographs import ldt_deveny
 from pypeit.spectrographs import magellan_fire
 from pypeit.spectrographs import magellan_mage
 from pypeit.spectrographs import mdm_osmos
@@ -27,24 +28,19 @@ from pypeit.spectrographs import not_alfosc
 from pypeit.spectrographs import p200_dbsp
 from pypeit.spectrographs import p200_tspec
 from pypeit.spectrographs import shane_kast
+from pypeit.spectrographs import soar_goodman
 from pypeit.spectrographs import tng_dolores
 from pypeit.spectrographs import vlt_fors
 from pypeit.spectrographs import vlt_xshooter
 from pypeit.spectrographs import vlt_sinfoni
 from pypeit.spectrographs import wht_isis
+from pypeit.spectrographs import ntt_efosc2
+
 
 # Build the list of names for the available spectrographs
-import numpy as np
-
-def all_subclasses(cls):
-    """
-    Thanks to:
-    https://stackoverflow.com/questions/3862310/how-to-find-all-the-subclasses-of-a-class-given-its-name
-    """
-    return set(cls.__subclasses__()).union(
-        [s for c in cls.__subclasses__() for s in all_subclasses(c)])
 
 def spectrograph_classes():
+    import numpy as np
     # Recursively collect all subclasses
     spec_c = np.array(list(all_subclasses(spectrograph.Spectrograph)))
     # Select spectrograph classes with a defined name; spectrographs without a
