@@ -5,6 +5,7 @@ Module for LRIS specific methods.
 """
 import glob
 import os
+import pdb
 
 from IPython import embed
 
@@ -15,6 +16,8 @@ import numpy as np
 from astropy.io import fits
 from astropy import time
 
+from linetools import utils as ltu
+
 from pypeit import msgs
 from pypeit import telescopes
 from pypeit import io
@@ -52,6 +55,8 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         # the data in the dev suite is unbinned.
         # JXP -- Increased to 6 arcsec.  I don't know how 2 (or 1!) could have worked.
         par['calibrations']['slitedges']['minimum_slit_length_sci'] = 6
+        # Remove slits that are too short
+        par['calibrations']['slitedges']['minimum_slit_length'] = 4.
         # 1D wavelengths
         par['calibrations']['wavelengths']['rms_threshold'] = 0.20  # Might be grism dependent
         # Set the default exposure time ranges for the frame typing
@@ -121,7 +126,7 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         self.meta['target'] = dict(ext=0, card='TARGNAME')
         self.meta['decker'] = dict(ext=0, card='SLITNAME')
         self.meta['binning'] = dict(card=None, compound=True)
-
+        # 
         self.meta['mjd'] = dict(ext=0, card='MJD-OBS')
         self.meta['exptime'] = dict(ext=0, card='ELAPTIME')
         self.meta['airmass'] = dict(ext=0, card='AIRMASS')
@@ -130,6 +135,14 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         self.meta['hatch'] = dict(ext=0, card='TRAPDOOR')
         # Red only, but grabbing here
         self.meta['dispangle'] = dict(ext=0, card='GRANGLE', rtol=1e-2)
+        self.meta['frameno'] = dict(ext=0, card='FRAMENO')
+        self.meta['instrument'] = dict(ext=0, card='INSTRUME')
+
+        # Extras for pypeit file
+        if self.name == 'keck_lris_red_mark4':
+            self.meta['amp'] = dict(ext=0, card='TAPLINES')
+        else:
+            self.meta['amp'] = dict(ext=0, card='NUMAMPS')
 
         # Lamps -- Have varied in time..
         for kk in range(12): # This needs to match the length of LAMPS below
@@ -152,10 +165,11 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         if meta_key == 'binning':
             binspatial, binspec = parse.parse_binning(headarr[0]['BINNING'])
             binning = parse.binning2string(binspec, binspatial)
+
             return binning
         elif 'lampstat' in meta_key:
             idx = int(meta_key[-2:])
-            curr_date = time.Time(headarr[0]['MJD-OBS'], format='mjd')
+            curr_date = time.Time(self.get_meta_value(headarr, 'mjd'), format='mjd')
             # Modern -- Assuming the change occurred with the new red detector
             t_newlamp = time.Time("2014-02-15", format='isot')  # LAMPS changed in Header
             if curr_date > t_newlamp:
@@ -195,6 +209,17 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         """
         return super().configuration_keys() + ['binning']
 
+    def pypeit_file_keys(self):
+        """
+        Define the list of keys to be output into a standard ``PypeIt`` file.
+
+        Returns:
+            :obj:`list`: The list of keywords in the relevant
+            :class:`~pypeit.metadata.PypeItMetaData` instance to print to the
+            :ref:`pypeit_file`.
+        """
+        return super().pypeit_file_keys() + ['frameno']
+
     def check_frame_type(self, ftype, fitstbl, exprng=None):
         """
         Check for frames of the provided type.
@@ -222,8 +247,11 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         if ftype == 'bias':
             return good_exp & self.lamps(fitstbl, 'off') & (fitstbl['hatch'] == 'closed')
         if ftype in ['pixelflat', 'trace', 'illumflat']:
+            # Allow for dome or internal
+            good_dome = self.lamps(fitstbl, 'dome') & (fitstbl['hatch'] == 'open')
+            good_internal = self.lamps(fitstbl, 'halogen') & (fitstbl['hatch'] == 'closed')
             # Flats and trace frames are typed together
-            return good_exp & self.lamps(fitstbl, 'dome') & (fitstbl['hatch'] == 'open')
+            return good_exp & (good_dome + good_internal)
         if ftype in ['pinhole', 'dark']:
             # Don't type pinhole or dark frames
             return np.zeros(len(fitstbl), dtype=bool)
@@ -256,17 +284,24 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
             # Check if all are off
             return np.all(np.array([ (fitstbl[k] == 'off') | (fitstbl[k] == 'None')
                                         for k in fitstbl.keys() if 'lampstat' in k]), axis=0)
-        if status == 'arcs':
+        elif status == 'arcs':
             # Check if any arc lamps are on
             arc_lamp_stat = [ 'lampstat{0:02d}'.format(i) for i in range(1,9) ]
             return np.any(np.array([ fitstbl[k] == 'on' for k in fitstbl.keys()
                                             if k in arc_lamp_stat]), axis=0)
-        if status == 'dome':
+        elif status == 'dome':
             # Check if any dome lamps are on
             # Warning 9, 10 are FEARGON and DEUTERI
             dome_lamp_stat = [ 'lampstat{0:02d}'.format(i) for i in range(9,13) ]
             return np.any(np.array([ fitstbl[k] == 'on' for k in fitstbl.keys()
                                             if k in dome_lamp_stat]), axis=0)
+        elif status == 'halogen':
+            halo_lamp_stat = ['lampstat12']
+            return np.any(np.array([ fitstbl[k] == 'on' for k in fitstbl.keys()
+                                            if k in halo_lamp_stat]), axis=0)
+        else:
+            msgs.error(f"Bad status option! {status}")
+
         raise ValueError('No implementation for status = {0}'.format(status))
 
     def get_rawimage(self, raw_file, det):
@@ -416,7 +451,7 @@ class KeckLRISSpectrograph(spectrograph.Spectrograph):
         # Need the exposure time
         exptime = hdu[self.meta['exptime']['ext']].header[self.meta['exptime']['card']]
         # Return
-        return self.get_detector_par(hdu, det if det is not None else 1), \
+        return self.get_detector_par(det if det is not None else 1, hdu=hdu), \
                 array.T, hdu, exptime, rawdatasec_img.T, oscansec_img.T
 
     def subheader_for_spec(self, row_fitstbl, raw_header, extra_header_cards=None,
@@ -463,18 +498,20 @@ class KeckLRISBSpectrograph(KeckLRISSpectrograph):
 
     name = 'keck_lris_blue'
     camera = 'LRISb'
+    header_name = 'LRISBLUE'
     supported = True
     comment = 'Blue camera; see :doc:`lris`'
     
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
@@ -482,7 +519,7 @@ class KeckLRISBSpectrograph(KeckLRISSpectrograph):
         """
         # Binning
         # TODO: Could this be detector dependent?
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict1 = dict(
@@ -514,11 +551,17 @@ class KeckLRISBSpectrograph(KeckLRISSpectrograph):
         detector_dicts = [detector_dict1, detector_dict2]
         detector = detector_container.DetectorContainer(**detector_dicts[det-1])
 
+        if hdu is None:
+            return detector
+
+        # TODO: I don't know how to handle the hack below for the auto-generated
+        # detector table...
+
         # Deal with number of amps
         namps = hdu[0].header['NUMAMPS']
         # The website does not give values for single amp per detector so we take the mean
         #   of the values provided
-        if namps == 2 or ((namps==4) & (len(hdu)==3)):  # Longslit readout mode is the latter.  This is a hack..
+        if namps == 2 or (namps==4 and len(hdu)==3):  # Longslit readout mode is the latter.  This is a hack..
             detector.numamplifiers = 1
             detector.gain = np.atleast_1d(np.mean(detector.gain))
             detector.ronoise = np.atleast_1d(np.mean(detector.ronoise))
@@ -684,6 +727,25 @@ class KeckLRISBOrigSpectrograph(KeckLRISBSpectrograph):
             if 'lampstat' in key:
                 self.meta.pop(key)
 
+    def get_detector_par(self, det, hdu=None):
+        """
+        Return metadata for the selected detector.
+
+        Args:
+            det (:obj:`int`):
+                1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
+
+        Returns:
+            :class:`~pypeit.images.detector_container.DetectorContainer`:
+            Object with the detector metadata.
+        """
+        detpar = super().get_detector_par(det, hdu=hdu)
+        detpar['specflip'] = True
+        return detpar
+
     def get_rawimage(self, raw_file, det):
         """
         Read raw images and generate a few other bits and pieces
@@ -741,10 +803,10 @@ class KeckLRISBOrigSpectrograph(KeckLRISBSpectrograph):
             msgs.error("Should not be here in keck_lris!")
 
         # Detector
-        detector_par = self.get_detector_par(hdul, det-1)
+        detector_par = self.get_detector_par(det-1, hdu=hdul)
 
-        # Flip the spectral axis
-        detector_par['specflip'] = True
+#        # Flip the spectral axis
+#        detector_par['specflip'] = True
 
         # Return
         return detector_par, image, hdul, elaptime, rawdatasec_img, oscansec_img
@@ -788,25 +850,28 @@ class KeckLRISRSpectrograph(KeckLRISSpectrograph):
     """
     name = 'keck_lris_red'
     camera = 'LRISr'
+    header_name = 'LRIS'
     supported = True
-    comment = 'Red camera; see :doc:`lris`'
+    comment = 'Red camera;  LBNL detector, 2kx4k; see :doc:`lris`'
     
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        # TODO: Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict1 = dict(
@@ -834,35 +899,45 @@ class KeckLRISRSpectrograph(KeckLRISSpectrograph):
             ronoise=np.atleast_1d([4.54, 4.62])
         ))
 
-        # Allow for post COVID detector issues
-        t2020_1 = time.Time("2020-06-30", format='isot')  # First run
-        t2020_2 = time.Time("2020-07-29", format='isot')  # Second run
-        date = time.Time(hdu[0].header['MJD-OBS'], format='mjd')
+        if hdu is not None:
+            # Allow for post COVID detector issues
+            t2020_1 = time.Time("2020-06-30", format='isot')  # First run
+            t2020_2 = time.Time("2020-07-29", format='isot')  # Second run
+            # Check for the new detector (Mark4) upgrade
+            t2021_upgrade = time.Time("2021-04-15", format='isot') 
+            date = time.Time(self.get_meta_value(self.get_headarr(hdu), 'mjd'), 
+                             format='mjd')
 
-        if date < t2020_1:
-            pass
-        elif date < t2020_2: # This is for the June 30 2020 run
-            msgs.warn("We are using LRISr gain/RN values based on WMKO estimates.")
-            detector_dict1['gain'] = np.atleast_1d([37.6])
-            detector_dict2['gain'] = np.atleast_1d([1.26])
-            detector_dict1['ronoise'] = np.atleast_1d([99.])
-            detector_dict2['ronoise'] = np.atleast_1d([5.2])
-        else: # This is the 2020 July 29 run
-            msgs.warn("We are using LRISr gain/RN values based on WMKO estimates.")
-            detector_dict1['gain'] = np.atleast_1d([1.45])
-            detector_dict2['gain'] = np.atleast_1d([1.25])
-            detector_dict1['ronoise'] = np.atleast_1d([4.47])
-            detector_dict2['ronoise'] = np.atleast_1d([4.75])
+            if date < t2020_1:
+                pass
+            elif date < t2020_2: # This is for the June 30 2020 run
+                msgs.warn("We are using LRISr gain/RN values based on WMKO estimates.")
+                detector_dict1['gain'] = np.atleast_1d([37.6])
+                detector_dict2['gain'] = np.atleast_1d([1.26])
+                detector_dict1['ronoise'] = np.atleast_1d([99.])
+                detector_dict2['ronoise'] = np.atleast_1d([5.2])
+            elif date > t2021_upgrade: 
+                # Note:  We are unlikely to trip this.  Other things probably failed first
+                msgs.error("This is the new detector.  Use keck_lris_red_mark4")
+            else: # This is the 2020 July 29 run
+                msgs.warn("We are using LRISr gain/RN values based on WMKO estimates.")
+                detector_dict1['gain'] = np.atleast_1d([1.45])
+                detector_dict2['gain'] = np.atleast_1d([1.25])
+                detector_dict1['ronoise'] = np.atleast_1d([4.47])
+                detector_dict2['ronoise'] = np.atleast_1d([4.75])
 
         # Instantiate
         detector_dicts = [detector_dict1, detector_dict2]
         detector = detector_container.DetectorContainer(**detector_dicts[det-1])
 
+        if hdu is None:
+            return detector
+
         # Deal with number of amps
         namps = hdu[0].header['NUMAMPS']
         # The website does not give values for single amp per detector so we take the mean
         #   of the values provided
-        if namps == 2 or ((namps==4) & (len(hdu)==3)):  # Longslit readout mode is the latter.  This is a hack..
+        if namps == 2 or (namps==4 and len(hdu)==3):  # Longslit readout mode is the latter.  This is a hack..
             detector.numamplifiers = 1
             # Long silt mode
             if hdu[0].header['AMPPSIZE'] == '[1:1024,1:4096]':
@@ -948,7 +1023,10 @@ class KeckLRISRSpectrograph(KeckLRISSpectrograph):
 
         # Wavelength calibrations
         if self.get_meta_value(scifile, 'dispname') == '400/8500':  # This is basically a reidentify
-            par['calibrations']['wavelengths']['reid_arxiv'] = 'keck_lris_red_400.fits'
+            if self.name == 'keck_lris_red_mark4':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'keck_lris_red_mark4_R400.fits'
+            else:
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'keck_lris_red_400.fits'
             par['calibrations']['wavelengths']['method'] = 'full_template'
             par['calibrations']['wavelengths']['sigdetect'] = 20.0
             par['calibrations']['wavelengths']['nsnippet'] = 1
@@ -998,7 +1076,7 @@ class KeckLRISRSpectrograph(KeckLRISSpectrograph):
             and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
             object.
         """
-        return super().configuration_keys() + ['dispangle']
+        return super().configuration_keys() + ['dispangle', 'amp', 'binning']
 
     def bpm(self, filename, det, shape=None, msbias=None):
         """
@@ -1050,6 +1128,189 @@ class KeckLRISRSpectrograph(KeckLRISSpectrograph):
 
         return bpm_img
 
+class KeckLRISRMark4Spectrograph(KeckLRISRSpectrograph):
+    """
+    Child to handle the new Mark4 detector
+    """
+    ndet = 1
+    name = 'keck_lris_red_mark4'
+    supported = True
+    comment = 'New Mark4 detector, circa Spring 2021; Supported setups = R400'
+
+    def init_meta(self):
+        super().init_meta()
+        # Over-ride a pair
+        self.meta['mjd'] = dict(ext=0, card='MJD')
+        self.meta['exptime'] = dict(ext=0, card='TELAPSE')
+
+    def get_detector_par(self, det, hdu=None):
+        """
+        Return metadata for the selected detector.
+
+        Args:
+            det (:obj:`int`):
+                1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
+
+        Returns:
+            :class:`~pypeit.images.detector_container.DetectorContainer`:
+            Object with the detector metadata.
+        """
+        # Binning
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
+
+        # Detector 1
+        detector_dict1 = dict(
+            binning=binning,
+            det=1,
+            dataext=0,
+            specaxis=0,
+            specflip=True,  
+            spatflip=False,
+            platescale=0.123,  # From the web page
+            darkcurr=0.0,
+            saturation=65535.,
+            nonlinear=0.76,
+            mincounts=-1e10,
+            numamplifiers=2,  # These are defaults but can modify below
+            gain=np.atleast_1d([1.61, 1.67*0.959]), # Assumes AMPMODE=HSPLIT,VUP;  Corrected by JXP using 2x1 binned flats
+            ronoise=np.atleast_1d([3.65, 3.52]),
+        )
+
+        if hdu is None:
+            return detector_container.DetectorContainer(**detector_dict1)
+
+        # Date of Mark4 installation
+        t2021_upgrade = time.Time("2021-04-15", format='isot') 
+        # TODO -- Update with the date we transitioned to the correct ones
+        t_gdhead = time.Time("2022-01-01", format='isot')  
+        date = time.Time(hdu[0].header['MJD'], format='mjd')
+
+        if date < t2021_upgrade:
+            msgs.error("This is not the Mark4 detector.  Use a different keck_lris_red spectrograph")
+
+        # Deal with the intermediate headers
+        if date < t_gdhead:
+            amp_mode = hdu[0].header['AMPMODE']
+            msgs.info("AMPMODE = {:s}".format(amp_mode))
+            # Load up translation dict
+            ampmode_translate_file = os.path.join(
+                resource_filename('pypeit', 'data'), 'spectrographs',
+                'keck_lris_red_mark4', 'dict_for_ampmode.json')
+            ampmode_translate_dict = ltu.loadjson(ampmode_translate_file)
+            # Load up the corrected header
+            swap_binning = f"{binning[-1]},{binning[0]}"  # LRIS convention is oppopsite ours
+            header_file = os.path.join(
+                resource_filename('pypeit', 'data'), 'spectrographs',
+                'keck_lris_red_mark4', 
+                f'header{ampmode_translate_dict[amp_mode]}_{swap_binning.replace(",","_")}.fits')
+            correct_header = fits.getheader(header_file)
+        else:
+            correct_header = hdu[0].header
+
+        # Deal with number of amps
+        detector_dict1['numamplifiers'] = correct_header['TAPLINES']
+
+        # The website does not give values for single amp per detector so we take the mean
+        #   of the values provided
+        if detector_dict1['numamplifiers'] == 2: 
+            pass
+        elif detector_dict1['numamplifiers'] == 4:
+            # From the web page on 2021-10-04 (L1, L2, U1, U2)
+            #  Corrected by JXP and SS using chk_lris_mark4_gain.py in the DevSuite
+            detector_dict1['gain'] = np.atleast_1d([1.710, 
+                                                    1.64*1.0245,  # L2
+                                                    1.61*1.0185,  # U1
+                                                    1.67*1.0052]) # U2
+            detector_dict1['ronoise'] = np.atleast_1d([3.64, 3.45, 3.65, 3.52])
+        else:
+            msgs.error("Did not see this namps coming..")
+
+        detector_dict1['datasec'] = []
+        detector_dict1['oscansec'] = []
+
+        # Parse which AMPS were used
+        used_amps = []
+        for amp in range(4):
+            if f'AMPNM{amp}' in correct_header.keys():
+                used_amps.append(amp)
+        # Check
+        assert detector_dict1['numamplifiers'] == len(used_amps)
+
+        # Reverse engenieering to translate LRIS DSEC, BSEC
+        #  into ones friendly for PypeIt...
+        binspec = int(binning[0])
+        binspatial = int(binning[-1])
+        
+        for iamp in used_amps:
+            # These are column, row
+            dsecs = correct_header[f'DSEC{iamp}'].split(',')
+            d_rows = [int(item) for item in dsecs[1][:-1].split(':')]
+            d_cols = [int(item) for item in dsecs[0][1:].split(':')]
+            bsecs = correct_header[f'BSEC{iamp}'].split(',')
+            o_rows = [int(item) for item in bsecs[1][:-1].split(':')]
+            o_cols = [int(item) for item in bsecs[0][1:].split(':')]
+
+            # Deal with binning (heaven help me!!)
+            d_rows = [str(item*binspec) if item != 1 else str(item) for item in d_rows]
+            o_rows = [str(item*binspec) if item != 1 else str(item) for item in o_rows]
+            d_cols = [str(item*binspatial) if item != 1 else str(item) for item in d_cols]
+            o_cols = [str(item*binspatial) if item != 1 else str(item) for item in o_cols]
+
+            # These are now row, column
+            #  And they need to be native!!  i.e. no binning accounted for..
+            detector_dict1['datasec'] += [f"[{':'.join(d_rows)},{':'.join(d_cols)}]"]
+            detector_dict1['oscansec'] += [f"[{':'.join(o_rows)},{':'.join(o_cols)}]"]
+
+        detector_dict1['datasec'] = np.array(detector_dict1['datasec'])
+        detector_dict1['oscansec'] = np.array(detector_dict1['oscansec'])
+
+        # Instantiate
+        detector = detector_container.DetectorContainer(**detector_dict1)
+
+        #print('Dict1:', detector_dict1)
+        #print('Binning:', binning)
+
+        # Return
+        return detector
+
+    def get_rawimage(self, raw_file, det):
+        """
+        Read raw images and generate a few other bits and pieces
+        that are key for image processing.
+
+        Over-ride standard get_rawimage() for LRIS
+
+        Parameters
+        ----------
+        raw_file : :obj:`str`
+            File to read
+        det : :obj:`int`
+            1-indexed detector to read
+
+        Returns
+        -------
+        detector_par : :class:`pypeit.images.detector_container.DetectorContainer`
+            Detector metadata parameters.
+        raw_img : `numpy.ndarray`_
+            Raw image for this detector.
+        hdu : `astropy.io.fits.HDUList`_
+            Opened fits file
+        exptime : :obj:`float`
+            Exposure time read from the file header
+        rawdatasec_img : `numpy.ndarray`_
+            Data (Science) section of the detector as provided by setting the
+            (1-indexed) number of the amplifier used to read each detector
+            pixel. Pixels unassociated with any amplifier are set to 0.
+        oscansec_img : `numpy.ndarray`_
+            Overscan section of the detector as provided by setting the
+            (1-indexed) number of the amplifier used to read each detector
+            pixel. Pixels unassociated with any amplifier are set to 0.
+        """
+        # Note:  There is no way we know to super super super
+        return spectrograph.Spectrograph.get_rawimage(self, raw_file, det)
 
 class KeckLRISROrigSpectrograph(KeckLRISRSpectrograph):
     """
@@ -1059,7 +1320,7 @@ class KeckLRISROrigSpectrograph(KeckLRISRSpectrograph):
     name = 'keck_lris_red_orig'
     camera = 'LRISr'
     supported = True
-    comment = 'Original detector; replaced in 20??; see :doc:`lris`'
+    comment = 'Original detector; replaced in 2009; see :doc:`lris`'
 
     @classmethod
     def default_pypeit_par(cls):
@@ -1077,22 +1338,24 @@ class KeckLRISROrigSpectrograph(KeckLRISRSpectrograph):
 
         return par
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        # TODO: Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict1 = dict(
@@ -1115,7 +1378,7 @@ class KeckLRISROrigSpectrograph(KeckLRISRSpectrograph):
         detector = detector_container.DetectorContainer(**detector_dict1)
 
         # Deal with number of amps
-        if hdu[0].header['NUMAMPS'] != 2:
+        if hdu is not None and hdu[0].header['NUMAMPS'] != 2:
             msgs.error("Did not see this namps coming..")
 
         # Return
@@ -1171,7 +1434,7 @@ class KeckLRISROrigSpectrograph(KeckLRISRSpectrograph):
         # Image info
         image, hdul, elaptime, rawdatasec_img, oscansec_img = get_orig_rawimage(raw_file)
         # Detector
-        detector_par = self.get_detector_par(hdul, det)
+        detector_par = self.get_detector_par(det, hdu=hdul)
         # Return
         return detector_par, image, hdul, elaptime, rawdatasec_img, oscansec_img
 
@@ -1258,7 +1521,6 @@ def lris_read_amp(inp, ext):
     # parse the DATASEC keyword to determine the size of the science region (unbinned)
     datasec = header['DATASEC']
     xdata1, xdata2, ydata1, ydata2 = np.array(parse.load_sections(datasec, fmt_iraf=False)).flatten()
-
     # grab the components...
     predata = temp[0:precol, :]
     # datasec appears to have the x value for the keywords that are zero
@@ -1388,7 +1650,6 @@ def get_orig_rawimage(raw_file, debug=False):
         oscansec_img[:, biascols] = iamp+1
         imagecols = np.arange(1024 // xbin) + iamp * 1024 // xbin
         rawdatasec_img[:,imagecols + namps*(prepix // xbin)] = iamp+1
-
     return image, hdul, float(head0['ELAPTIME']), \
            rawdatasec_img, oscansec_img
 
